@@ -5,7 +5,7 @@
 
     Build input files for the Qt collection generator.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2014 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -14,12 +14,13 @@ import re
 import codecs
 import posixpath
 from os import path
-from cgi import escape
 
 from docutils import nodes
 
 from sphinx import addnodes
 from sphinx.builders.html import StandaloneHTMLBuilder
+from sphinx.util import force_decode
+from sphinx.util.pycompat import htmlescape
 
 
 _idpattern = re.compile(
@@ -130,16 +131,16 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         for indexname, indexcls, content, collapse in self.domain_indices:
             item = section_template % {'title': indexcls.localname,
                                        'ref': '%s.html' % indexname}
-            sections.append((' ' * 4 * 4 + item).encode('utf-8'))
+            sections.append(' ' * 4 * 4 + item)
         # sections may be unicode strings or byte strings, we have to make sure
-        # they are all byte strings before joining them
+        # they are all unicode strings before joining them
         new_sections = []
         for section in sections:
-            if isinstance(section, unicode):
-                new_sections.append(section.encode('utf-8'))
+            if not isinstance(section, unicode):
+                new_sections.append(force_decode(section, None))
             else:
                 new_sections.append(section)
-        sections = u'\n'.encode('utf-8').join(new_sections)
+        sections = u'\n'.join(new_sections)
 
         # keywords
         keywords = []
@@ -147,7 +148,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         for (key, group) in index:
             for title, (refs, subitems) in group:
                 keywords.extend(self.build_keywords(title, refs, subitems))
-        keywords = '\n'.join(keywords)
+        keywords = u'\n'.join(keywords)
 
         # files
         if not outdir.endswith(os.sep):
@@ -164,7 +165,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
                        fn.endswith('.html'):
                     filename = path.join(root, fn)[olen:]
                     projectfiles.append(file_template %
-                                        {'filename': escape(filename)})
+                                        {'filename': htmlescape(filename)})
         projectfiles = '\n'.join(projectfiles)
 
         # it seems that the "namespace" may not contain non-alphanumeric
@@ -179,12 +180,12 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         f = codecs.open(path.join(outdir, outname+'.qhp'), 'w', 'utf-8')
         try:
             f.write(project_template % {
-                'outname': escape(outname),
-                'title': escape(self.config.html_title),
-                'version': escape(self.config.version),
-                'project': escape(self.config.project),
-                'namespace': escape(nspace),
-                'masterdoc': escape(self.config.master_doc),
+                'outname': htmlescape(outname),
+                'title': htmlescape(self.config.html_title),
+                'version': htmlescape(self.config.version),
+                'project': htmlescape(self.config.project),
+                'namespace': htmlescape(nspace),
+                'masterdoc': htmlescape(self.config.master_doc),
                 'sections': sections,
                 'keywords': keywords,
                 'files': projectfiles})
@@ -199,10 +200,10 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         f = codecs.open(path.join(outdir, outname+'.qhcp'), 'w', 'utf-8')
         try:
             f.write(collection_template % {
-                'outname': escape(outname),
-                'title': escape(self.config.html_short_title),
-                'homepage': escape(homepage),
-                'startpage': escape(startpage)})
+                'outname': htmlescape(outname),
+                'title': htmlescape(self.config.html_short_title),
+                'homepage': htmlescape(homepage),
+                'startpage': htmlescape(startpage)})
         finally:
             f.close()
 
@@ -220,14 +221,14 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         return True
 
     def write_toc(self, node, indentlevel=4):
+        # XXX this should return a Unicode string, not a bytestring
         parts = []
         if self.isdocnode(node):
             refnode = node.children[0][0]
             link = refnode['refuri']
-            title = escape(refnode.astext()).replace('"','&quot;')
-            item = '<section title="%(title)s" ref="%(ref)s">' % {
-                                                                'title': title,
-                                                                'ref': link}
+            title = htmlescape(refnode.astext()).replace('"', '&quot;')
+            item = '<section title="%(title)s" ref="%(ref)s">' % \
+                {'title': title, 'ref': link}
             parts.append(' '*4*indentlevel + item)
             for subnode in node.children[1]:
                 parts.extend(self.write_toc(subnode, indentlevel+1))
@@ -237,7 +238,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
                 parts.extend(self.write_toc(subnode, indentlevel))
         elif isinstance(node, nodes.reference):
             link = node['refuri']
-            title = escape(node.astext()).replace('"','&quot;')
+            title = htmlescape(node.astext()).replace('"','&quot;')
             item = section_template % {'title': title, 'ref': link}
             item = u' ' * 4 * indentlevel + item
             parts.append(item.encode('ascii', 'xmlcharrefreplace'))
@@ -274,7 +275,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
     def build_keywords(self, title, refs, subitems):
         keywords = []
 
-        title = escape(title)
+        title = htmlescape(title)
 #        if len(refs) == 0: # XXX
 #            write_param('See Also', title)
         if len(refs) == 1:
